unit main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Buttons, ExtDlgs, ComCtrls, Spin, Clipbrd,
  ImgList;

type
  TfrmMain = class(TForm)
    pcMain: TPageControl;
    tsInput: TTabSheet;
    tsOutput: TTabSheet;
    fdText: TFontDialog;
    otfdtext: TOpenTextFileDialog;
    stfdText: TSaveTextFileDialog;
    mOutput: TMemo;
    bbtnConvertText: TBitBtn;
    bbtnSaveResult: TBitBtn;
    bbtnRetry: TBitBtn;
    Label1: TLabel;
    edChars: TEdit;
    lblWidth: TLabel;
    lblHeight: TLabel;
    seWidth: TSpinEdit;
    seHeight: TSpinEdit;
    rbShuffle: TRadioButton;
    rbContinue: TRadioButton;
    bbtnResultCopy: TBitBtn;
    cbRender: TCheckBox;
    imglTabs: TImageList;
    pcInput: TPageControl;
    tsText: TTabSheet;
    mText2Convert: TMemo;
    bbtnFont: TBitBtn;
    bbtnSaveText: TBitBtn;
    bbtnTextLoad: TBitBtn;
    sbTextRight: TSpeedButton;
    sbTextCenter: TSpeedButton;
    sbTextLeft: TSpeedButton;
    lblText2Convert: TLabel;
    tsImage: TTabSheet;
    pnlImage: TPanel;
    imgInput: TImage;
    opdInput: TOpenPictureDialog;
    procedure bbtnTextLoadClick(Sender: TObject);
    procedure bbtnFontClick(Sender: TObject);
    procedure bbtnSaveTextClick(Sender: TObject);
    procedure bbtnConvertTextClick(Sender: TObject);
    procedure mText2ConvertChange(Sender: TObject);
    procedure pcMainChanging(Sender: TObject; var AllowChange: Boolean);
    procedure sbTextLeftClick(Sender: TObject);
    procedure sbTextCenterClick(Sender: TObject);
    procedure sbTextRightClick(Sender: TObject);
    procedure bbtnRetryClick(Sender: TObject);
    procedure bbtnSaveResultClick(Sender: TObject);
    procedure bbtnResultCopyClick(Sender: TObject);
    procedure imgInputClick(Sender: TObject);
    procedure tsOutputShow(Sender: TObject);
  private
    { Dclarations prives }
    procedure ConvertText;
    procedure ConvertImage;
    procedure ConvertToText(bmp: TBitmap);
    procedure SaveIt(who: TMemo);
    procedure CanProcess;
  public
    { Dclarations publiques }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

function Max(const A, B: Integer): Integer;
begin
  if A > B then
    Result := A
  else
    Result := B;
end;

procedure TfrmMain.bbtnFontClick(Sender: TObject);
begin
  fdText.Font := mText2Convert.Font;
  if fdText.Execute then
  begin
    mText2Convert.Font       := fdText.Font;
    mText2Convert.Font.Color := clWindowText;
  end;
end;

procedure TfrmMain.bbtnSaveTextClick(Sender: TObject);
begin
  SaveIt(mText2Convert);
end;

procedure TfrmMain.bbtnTextLoadClick(Sender: TObject);
begin
  if otfdtext.Execute then
  begin
    try
      mText2Convert.Lines.LoadFromFile(otfdtext.FileName);
    except
      MessageDlg('Erreur pendant le chargement', mtWarning, [mbOK], 0);
      mText2Convert.Clear;
    end;
  end;
end;

procedure TfrmMain.bbtnSaveResultClick(Sender: TObject);
begin
  SaveIt(mOutput);
end;

procedure TfrmMain.bbtnResultCopyClick(Sender: TObject);
begin
  Clipboard.AsText := mOutput.Text;
end;

procedure TfrmMain.bbtnRetryClick(Sender: TObject);
begin
  pcMain.ActivePage := tsInput;
  mText2Convert.SetFocus;
end;

procedure TfrmMain.bbtnConvertTextClick(Sender: TObject);
begin
  pcMain.ActivePage := tsOutput;
end;

procedure TfrmMain.mText2ConvertChange(Sender: TObject);
begin
  CanProcess;
end;

procedure TfrmMain.pcMainChanging(Sender: TObject; var AllowChange: Boolean);
begin
  AllowChange := (((pcInput.ActivePage = tsText)
                AND
              (bbtnConvertText.Enabled))
      OR
              ((pcInput.ActivePage = tsImage)
                AND
              (imgInput.Picture.Bitmap.PixelFormat = pf1bit)));
end;

procedure TfrmMain.SaveIt(who: TMemo);
begin
  // Sauvegarde le texte rsultat dans un fichier texte
  if stfdtext.Execute then
  begin
    try
      who.Lines.SaveToFile(stfdtext.FileName);
    except
      MessageDlg('Erreur pendant la sauvegarde', mtWarning, [mbOK], 0);
    end;
  end;
end;

procedure TfrmMain.CanProcess;
begin
  bbtnConvertText.Enabled := ((pcInput.ActivePage = tsText)
                              and (Length(trim(mText2Convert.Text)) > 0)
                              and (Length(trim(edChars.Text)) > 0)
                              and (seWidth.Value > 0)
                              and (seHeight.Value > 0))
                              OR
                              ((pcInput.ActivePage = tsImage)
                              and (imgInput.Picture.Bitmap.PixelFormat = pf1bit));
  tsOutput.Enabled := bbtnConvertText.Enabled;
end;

procedure TfrmMain.sbTextCenterClick(Sender: TObject);
begin
  mText2Convert.Alignment := taCenter;
end;

procedure TfrmMain.sbTextLeftClick(Sender: TObject);
begin
  mText2Convert.Alignment := taLeftJustify;
end;

procedure TfrmMain.sbTextRightClick(Sender: TObject);
begin
  mText2Convert.Alignment := taRightJustify;
end;

procedure TfrmMain.tsOutputShow(Sender: TObject);
begin
  if (pcInput.ActivePage = tsText) then
    ConvertText
  else
    ConvertImage;

    beep;
end;

procedure TfrmMain.ConvertImage;
var
  curseur    : TCursor;
begin
  curseur   := Screen.Cursor;

  // Cration du bitmap qui va stocker un image temporaire contenant le texte
  try
    Screen.Cursor := crHourGlass;
    mOutput.Clear;
    mOutput.Lines.BeginUpdate;

    // Transforme l'image charge en ASCII Art
    ConvertToText(imgInput.Picture.Bitmap);

  finally
    mOutput.Lines.EndUpdate;
    Screen.Cursor := curseur;
  end;
end;

procedure TfrmMain.ConvertText;
var
  curseur    : TCursor;
  bmp        : TBitmap;
  loop       : Integer;
  higher     : Integer;
  deltaAlign : Integer;
begin
  curseur   := Screen.Cursor;

  // Cration du bitmap qui va stocker un image temporaire contenant le texte
  bmp     := TBitmap.Create;
  try
    Screen.Cursor := crHourGlass;
    mOutput.Clear;
    mOutput.Lines.BeginUpdate;

    bmp.Canvas.Font := mText2Convert.Font;
    bmp.Canvas.Brush.Style := bsClear;
    bmp.Canvas.Brush.Color := clWhite;
    bmp.Canvas.Pen.Mode := pmXor;
    bmp.Canvas.Pen.Color := clBlack;

    // Calcul de la largeur max de texte
    higher := 0;
    for loop := 0 to mText2Convert.Lines.Count - 1 do
    begin
      higher := Max(higher, bmp.Canvas.TextWidth(mText2Convert.Lines[loop]));
    end;
    bmp.Width := higher;

    // Calcul de la hauteur globale
    higher := 0;
    for loop := 0 to mText2Convert.Lines.Count - 1 do
    begin
      higher := higher + bmp.Canvas.TextHeight(mText2Convert.Lines[loop]);
    end;
    bmp.Height := higher;

    // Efface l'image
    bmp.Canvas.FillRect(Rect(0, 0, bmp.Width, bmp.Height));

    // Ecrit le texte dans l'image
    higher := 0;
    for loop := 0 to mText2Convert.Lines.Count - 1 do
    begin
      case mText2Convert.Alignment of
        taRightJustify:
        begin
          deltaAlign := bmp.Width - bmp.Canvas.TextWidth(mText2Convert.Lines[loop]);
        end;
        taCenter:
        begin
          deltaAlign := (bmp.Width - bmp.Canvas.TextWidth(mText2Convert.Lines[loop])) div 2;
        end;
        else
        begin
          deltaAlign := 0;
        end;
      end;
      bmp.Canvas.TextOut(deltaAlign, higher, mText2Convert.Lines[loop]);
      higher := higher + bmp.Canvas.TextHeight(mText2Convert.Lines[loop]);
    end;

    // Transforme l'image gnre en ASCII Art
    ConvertToText(bmp);

  finally
    mOutput.Lines.EndUpdate;
    bmp.Free;
    Screen.Cursor := curseur;
  end;
end;

procedure TfrmMain.ConvertToText(bmp: TBitmap);
var
  bmpGS      : TBitmap;
  row, col   : Integer;
  mRow, mCol : Integer;
  oneRow     : String;
  contIndex  : Integer;

  function getChar: Char;
  begin
    if rbShuffle.Checked then
    begin
      // On prend un caractre au hasard
      Result := edChars.Text[Random(Length(edChars.Text)) + 1];
    end
    else
    begin
      // Chaque caractre  tour de role
      Result := edChars.Text[contIndex];
      Inc(contIndex);
      if contIndex>Length(edChars.Text) then
        contIndex := 1;
    end;
    // Pour empcher l'utilisateur d'utiliser des caractres interdits
    if ((Ord(Result) < 32) OR (128 < Ord(Result))) then
      Result := 'W';
  end;
begin
  // Pour le "squenage" des caractres  utiliser
  contIndex := 1;

  // Cration du bitmap qui va stocker un image temporaire contenant le texte
  bmpGS   := TBitmap.Create;
  try
    // Redimensionne l'image pour obtenir le nombre de caractres souhait
    bmpGS.Width  := seWidth.Value;
    bmpGS.Height := seHeight.Value;
    bmpGS.Canvas.Brush.Color := clWhite;
    bmpGS.Canvas.FillRect(Rect(0, 0, bmpGS.Width, bmpGS.Height));
    bmpGS.Canvas.StretchDraw(Rect(0, 0, bmpGS.Width, bmpGS.Height), bmp);

    mRow := bmpGS.Height -1;
    mCol := bmpGS.Width  -1;
    // Parse l'image pour gnrer l'ASCII ART
    for row := 0 to mRow do
    begin
      oneRow := '';
      for col := 0 to mCol do
      begin
        if ((bmpGS.Canvas.Pixels[col, row] = clWhite) XOR (Not cbRender.Checked)) then
          oneRow := oneRow + getChar
        else
          oneRow := oneRow + ' ';
      end;
      mOutput.Lines.Add(oneRow);
    end;
  finally
    bmpGS.Free;
  end;
end;

procedure TfrmMain.imgInputClick(Sender: TObject);
begin
  if opdInput.Execute then
  begin
    try
      imgInput.Picture.LoadFromFile(opdInput.FileName);
      // Deux couleurs obligatoires
      imgInput.Picture.Bitmap.PixelFormat := pf1bit;
      // MAJ des contrles
      CanProcess;
    except
      MessageDlg('Erreur pendant le chargement', mtWarning, [mbOK], 0);
    end;
  end;
end;

end.
